@@ -3,6 +3,7 @@ package custom
 import (
 	"bytes"
 	"context"
+	"encoding/json"
 	"fmt"
 	"io"
 	"os"
@@ -41,8 +42,9 @@ type executorTestCase struct {
 		args []string,
 		options process.CommandOptions,
 	)
-	assertOutput  func(t *testing.T, output string)
-	expectedError string
+	assertOutput   func(t *testing.T, output string)
+	assertExecutor func(t *testing.T, e *executor)
+	expectedError  string
 }
 
 func getRunnerConfig(custom *common.CustomConfig) common.RunnerConfig {
@@ -268,6 +270,9 @@ func TestExecutor_Prepare(t *testing.T) {
 				assert.Equal(t, "/builds/project-0", b.BuildDir)
 				assert.Equal(t, "/cache/project-0", b.CacheDir)
 			},
+			assertExecutor: func(t *testing.T, e *executor) {
+				assert.Nil(t, e.jobEnv)
+			},
 		},
 		"custom executor set with ConfigExec with undefined builds_dir": {
 			config: getRunnerConfig(&common.CustomConfig{
@@ -420,6 +425,75 @@ func TestExecutor_Prepare(t *testing.T) {
 			},
 			expectedError: "test-error",
 		},
+		"custom executor set with valid job_env": {
+			config: getRunnerConfig(&common.CustomConfig{
+				RunExec:    "bash",
+				ConfigExec: "echo",
+			}),
+			commandStdoutContent: `{
+				"builds_dir": "/some/build/directory",
+				"job_env": {
+					"FOO": "Hello",
+					"BAR": "World"
+				}
+			}`,
+			commandErr: nil,
+			assertCommandFactory: func(
+				t *testing.T,
+				tt executorTestCase,
+				ctx context.Context,
+				executable string,
+				args []string,
+				options process.CommandOptions,
+			) {
+				assert.Equal(t, tt.config.Custom.ConfigExec, executable)
+			},
+			assertBuild: func(t *testing.T, b *common.Build) {
+				assert.Equal(t, "/some/build/directory/project-0", b.BuildDir)
+			},
+			assertExecutor: func(t *testing.T, e *executor) {
+				assert.Len(t, e.jobEnv, 2)
+				require.Contains(t, e.jobEnv, "FOO")
+				assert.Equal(t, "Hello", e.jobEnv["FOO"])
+				require.Contains(t, e.jobEnv, "BAR")
+				assert.Equal(t, "World", e.jobEnv["BAR"])
+			},
+		},
+		"custom executor set with valid job_env, verify variable order and prefix": {
+			config: getRunnerConfig(&common.CustomConfig{
+				RunExec:     "run-executable",
+				ConfigExec:  "config-executable",
+				PrepareExec: "prepare-executable",
+				PrepareArgs: []string{"test"},
+			}),
+			commandStdoutContent: `{
+				"builds_dir": "/some/build/directory",
+				"job_env": {
+					"FOO": "Hello"
+				}
+			}`,
+			commandErr: nil,
+			assertCommandFactory: func(
+				t *testing.T,
+				tt executorTestCase,
+				ctx context.Context,
+				executable string,
+				args []string,
+				options process.CommandOptions,
+			) {
+				if executable != "prepare-executable" {
+					return
+				}
+
+				require.True(t, len(options.Env) >= 2, "options.Env must contain 2 elements or more")
+				assert.Equal(t, "FOO=Hello", options.Env[0], "first env var must be FOO")
+				assert.True(
+					t,
+					strings.HasPrefix(options.Env[1], "CUSTOM_ENV_"),
+					"must be followed by CUSTOM_ENV_* variables",
+				)
+			},
+		},
 	}
 
 	for testName, tt := range tests {
@@ -435,6 +509,10 @@ func TestExecutor_Prepare(t *testing.T) {
 				tt.assertBuild(t, e.Build)
 			}
 
+			if tt.assertExecutor != nil {
+				tt.assertExecutor(t, e)
+			}
+
 			if tt.expectedError == "" {
 				assert.NoError(t, err)
 
@@ -511,6 +589,32 @@ func TestExecutor_Cleanup(t *testing.T) {
 				assert.Contains(t, output, "WARNING: Cleanup script failed: test-error")
 			},
 		},
+		"custom executor set with valid job_env, verify variable order and prefix": {
+			config: getRunnerConfig(&common.CustomConfig{
+				RunExec:     "bash",
+				CleanupExec: "echo",
+				CleanupArgs: []string{"test"},
+			}),
+			adjustExecutor: func(t *testing.T, e *executor) {
+				e.jobEnv = map[string]string{"FOO": "Hello"}
+			},
+			assertCommandFactory: func(
+				t *testing.T,
+				tt executorTestCase,
+				ctx context.Context,
+				executable string,
+				args []string,
+				options process.CommandOptions,
+			) {
+				require.True(t, len(options.Env) >= 2, "options.Env must contain 2 elements or more")
+				assert.Equal(t, "FOO=Hello", options.Env[0], "first env var must be FOO")
+				assert.True(
+					t,
+					strings.HasPrefix(options.Env[1], "CUSTOM_ENV_"),
+					"must be followed by CUSTOM_ENV_* variables",
+				)
+			},
+		},
 	}
 
 	for testName, tt := range tests {
@@ -518,6 +622,11 @@ func TestExecutor_Cleanup(t *testing.T) {
 			defer mockCommandFactory(t, tt)()
 
 			e, out := prepareExecutorForCleanup(t, tt)
+
+			if tt.adjustExecutor != nil {
+				tt.adjustExecutor(t, e)
+			}
+
 			e.Cleanup()
 
 			assertOutput(t, tt, out)
@@ -580,6 +689,30 @@ func TestExecutor_Run(t *testing.T) {
 			},
 			expectedError: "test-error",
 		},
+		"custom executor set with valid job_env, verify variable order and prefix": {
+			config: getRunnerConfig(&common.CustomConfig{
+				RunExec: "bash",
+			}),
+			adjustExecutor: func(t *testing.T, e *executor) {
+				e.jobEnv = map[string]string{"FOO": "Hello"}
+			},
+			assertCommandFactory: func(
+				t *testing.T,
+				tt executorTestCase,
+				ctx context.Context,
+				executable string,
+				args []string,
+				options process.CommandOptions,
+			) {
+				require.True(t, len(options.Env) >= 2, "options.Env must contain 2 elements or more")
+				assert.Equal(t, "FOO=Hello", options.Env[0], "first env var must be FOO")
+				assert.True(
+					t,
+					strings.HasPrefix(options.Env[1], "CUSTOM_ENV_"),
+					"must be followed by CUSTOM_ENV_* variables",
+				)
+			},
+		},
 	}
 
 	for testName, tt := range tests {
@@ -711,3 +844,175 @@ func TestExecutor_Env(t *testing.T) {
 		})
 	}
 }
+
+func TestExecutor_ServicesEnv(t *testing.T) {
+	const CIJobServicesEnv = "CUSTOM_ENV_CI_JOB_SERVICES"
+
+	runnerConfig := getRunnerConfig(&common.CustomConfig{
+		RunExec:     "bash",
+		PrepareExec: "echo",
+		CleanupExec: "bash",
+	})
+
+	adjustExecutorServices := func(services common.Services) func(t *testing.T, e *executor) {
+		return func(t *testing.T, e *executor) {
+			e.Build.Services = services
+		}
+	}
+
+	assertEnvValue := func(expectedServices []jsonService) func(
+		t *testing.T,
+		tt executorTestCase,
+		ctx context.Context,
+		executable string,
+		args []string,
+		options process.CommandOptions,
+	) {
+		return func(
+			t *testing.T,
+			tt executorTestCase,
+			ctx context.Context,
+			executable string,
+			args []string,
+			options process.CommandOptions,
+		) {
+			for _, env := range options.Env {
+				pair := strings.Split(env, "=")
+				if pair[0] == CIJobServicesEnv {
+					expectedServicesSerialized, _ := json.Marshal(expectedServices)
+
+					assert.Equal(t, string(expectedServicesSerialized), pair[1])
+					break
+				}
+			}
+		}
+	}
+
+	assertEmptyEnv := func() func(
+		t *testing.T,
+		tt executorTestCase,
+		ctx context.Context,
+		executable string,
+		args []string,
+		options process.CommandOptions,
+	) {
+		return func(
+			t *testing.T,
+			tt executorTestCase,
+			ctx context.Context,
+			executable string,
+			args []string,
+			options process.CommandOptions,
+		) {
+			for _, env := range options.Env {
+				pair := strings.Split(env, "=")
+				if pair[0] == CIJobServicesEnv {
+					assert.Equal(t, "", pair[1])
+					break
+				}
+			}
+		}
+	}
+
+	tests := map[string]executorTestCase{
+		"returns only name when service name is the only definition": {
+			config: runnerConfig,
+			adjustExecutor: adjustExecutorServices(common.Services{
+				{
+					Name: "ruby:latest",
+				},
+			}),
+			assertCommandFactory: assertEnvValue(
+				[]jsonService{
+					{
+						Name:       "ruby:latest",
+						Alias:      "",
+						Entrypoint: nil,
+						Command:    nil,
+					},
+				},
+			),
+		},
+		"returns full service definition": {
+			config: runnerConfig,
+			adjustExecutor: adjustExecutorServices(common.Services{
+				{
+					Name:       "ruby:latest",
+					Alias:      "henk-ruby",
+					Entrypoint: []string{"path", "to", "entrypoint"},
+					Command:    []string{"path", "to", "command"},
+				},
+			}),
+			assertCommandFactory: assertEnvValue(
+				[]jsonService{
+					{
+						Name:       "ruby:latest",
+						Alias:      "henk-ruby",
+						Entrypoint: []string{"path", "to", "entrypoint"},
+						Command:    []string{"path", "to", "command"},
+					},
+				},
+			),
+		},
+		"returns both simple and full service definitions": {
+			config: runnerConfig,
+			adjustExecutor: adjustExecutorServices(common.Services{
+				{
+					Name:       "python:latest",
+					Alias:      "henk-python",
+					Entrypoint: []string{"entrypoint.sh"},
+					Command:    []string{"command --test"},
+				},
+				{
+					Name: "python:alpine",
+				},
+			}),
+			assertCommandFactory: assertEnvValue(
+				[]jsonService{
+					{
+						Name:       "python:latest",
+						Alias:      "henk-python",
+						Entrypoint: []string{"entrypoint.sh"},
+						Command:    []string{"command --test"},
+					},
+					{
+						Name:       "python:alpine",
+						Alias:      "",
+						Entrypoint: nil,
+						Command:    nil,
+					},
+				},
+			),
+		},
+		"does not create env CI_JOB_SERVICES": {
+			config:               runnerConfig,
+			adjustExecutor:       adjustExecutorServices(common.Services{}),
+			assertCommandFactory: assertEmptyEnv(),
+		},
+	}
+
+	for tn, tt := range tests {
+		t.Run(tn, func(t *testing.T) {
+			defer mockCommandFactory(t, tt)()
+
+			e, options, _ := prepareExecutor(t, tt)
+			e.Config = *options.Config
+			e.Build = options.Build
+			e.Trace = options.Trace
+			e.BuildLogger = common.NewBuildLogger(e.Trace, e.Build.Log())
+			if tt.adjustExecutor != nil {
+				tt.adjustExecutor(t, e)
+			}
+
+			err := e.Prepare(options)
+			assert.NoError(t, err)
+
+			err = e.Run(common.ExecutorCommand{
+				Context: context.Background(),
+			})
+			assert.NoError(t, err)
+
+			e.Cleanup()
+		})
+	}
+}
