@@ -166,7 +166,7 @@ func getRunnerConfigForOS(t *testing.T) *common.RunnerConfig {
 			Shell:    shell,
 			Docker: &common.DockerConfig{
 				Image:      image,
-				PullPolicy: common.PullPolicyIfNotPresent,
+				PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 			},
 		},
 		RunnerCredentials: common.RunnerCredentials{
@@ -293,7 +293,7 @@ func TestDockerCommandNoRootImage(t *testing.T) {
 			RunnerSettings: common.RunnerSettings{
 				Executor: "docker",
 				Docker: &common.DockerConfig{
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -327,7 +327,7 @@ func TestDockerCommandWithAllowedImagesRun(t *testing.T) {
 					AllowedImages:   []string{common.TestAlpineImage},
 					AllowedServices: []string{common.TestDockerDindImage},
 					Privileged:      true,
-					PullPolicy:      common.PullPolicyIfNotPresent,
+					PullPolicy:      common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -388,7 +388,7 @@ func TestDockerCommandDisableEntrypointOverwrite(t *testing.T) {
 						Docker: &common.DockerConfig{
 							Privileged:                 true,
 							Image:                      common.TestAlpineImage,
-							PullPolicy:                 common.PullPolicyIfNotPresent,
+							PullPolicy:                 common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 							DisableEntrypointOverwrite: test.disabled,
 						},
 					},
@@ -437,7 +437,7 @@ func TestDockerCommandMissingImage(t *testing.T) {
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	assert.IsType(t, &common.BuildError{}, err)
+	assert.ErrorIs(t, err, &common.BuildError{})
 
 	contains := "repository does not exist"
 	if isDockerOlderThan17_07(t) {
@@ -455,7 +455,7 @@ func TestDockerCommandMissingTag(t *testing.T) {
 
 	err := build.Run(&common.Config{}, &common.Trace{Writer: os.Stdout})
 	require.Error(t, err)
-	assert.IsType(t, &common.BuildError{}, err)
+	assert.ErrorIs(t, err, &common.BuildError{})
 	assert.Contains(t, err.Error(), "not found")
 }
 
@@ -498,7 +498,7 @@ func TestDockerCommandTwoServicesFromOneImage(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -533,7 +533,7 @@ func TestDockerCommandOutput(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -576,7 +576,7 @@ func TestDockerPrivilegedServiceAccessingBuildsFolder(t *testing.T) {
 					Executor: "docker",
 					Docker: &common.DockerConfig{
 						Image:      common.TestAlpineImage,
-						PullPolicy: common.PullPolicyIfNotPresent,
+						PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 						Privileged: true,
 					},
 				},
@@ -620,7 +620,7 @@ func getTestDockerJob(t *testing.T) *common.Build {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					Privileged: true,
 				},
 			},
@@ -696,16 +696,6 @@ func TestDockerExtendedConfigurationFromJob(t *testing.T) {
 	}
 }
 
-func runTestJobWithOutput(t *testing.T, build *common.Build) (output string) {
-	var buffer bytes.Buffer
-
-	err := build.Run(&common.Config{}, &common.Trace{Writer: &buffer})
-	assert.NoError(t, err)
-
-	output = buffer.String()
-	return
-}
-
 func TestCacheInContainer(t *testing.T) {
 	test.SkipIfGitLabCIOn(t, test.OSWindows)
 	helpers.SkipIntegrationTests(t, "docker", "info")
@@ -736,7 +726,7 @@ func TestCacheInContainer(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					Volumes:    []string{"/cache"},
 				},
 			},
@@ -748,7 +738,8 @@ func TestCacheInContainer(t *testing.T) {
 	skipCacheUpload := "Not uploading cache key due to policy"
 
 	// The first job lacks any cache to pull, but tries to both pull and push
-	output := runTestJobWithOutput(t, build)
+	output, err := buildtest.RunBuildReturningOutput(t, build)
+	require.NoError(t, err)
 	assert.Regexp(t, cacheNotPresentRE, output, "First job execution should not have cached data")
 	assert.NotContains(
 		t,
@@ -767,7 +758,8 @@ func TestCacheInContainer(t *testing.T) {
 
 	// pull-only jobs should skip the push step
 	build.JobResponse.Cache[0].Policy = common.CachePolicyPull
-	output = runTestJobWithOutput(t, build)
+	output, err = buildtest.RunBuildReturningOutput(t, build)
+	require.NoError(t, err)
 	assert.NotRegexp(t, cacheNotPresentRE, output, "Second job execution should have cached data")
 	assert.NotContains(
 		t,
@@ -786,10 +778,21 @@ func TestCacheInContainer(t *testing.T) {
 
 	// push-only jobs should skip the pull step
 	build.JobResponse.Cache[0].Policy = common.CachePolicyPush
-	output = runTestJobWithOutput(t, build)
+	output, err = buildtest.RunBuildReturningOutput(t, build)
+	require.NoError(t, err)
 	assert.Regexp(t, cacheNotPresentRE, output, "Third job execution should not have cached data")
 	assert.Contains(t, output, skipCacheDownload, "Cache download be skipped with policy: push")
 	assert.NotContains(t, output, skipCacheUpload, "Cache upload should be performed with policy: push")
+
+	// For failed job it should push cache as well.
+	build.JobResponse.Cache[0].Policy = common.CachePolicyPullPush
+	build.JobResponse.Cache[0].When = common.CacheWhenAlways
+	build.JobResponse.Steps[0].Script = append(build.JobResponse.Steps[0].Script, "exit 1")
+	output, err = buildtest.RunBuildReturningOutput(t, build)
+	require.Error(t, err)
+	assert.NotRegexp(t, cacheNotPresentRE, output, "Second job execution should have cached data")
+	assert.Contains(t, output, "Saving cache for failed job")
+	assert.Contains(t, output, "Created cache")
 }
 
 func TestDockerImageNameFromVariable(t *testing.T) {
@@ -812,7 +815,7 @@ func TestDockerImageNameFromVariable(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:           common.TestAlpineImage,
-					PullPolicy:      common.PullPolicyIfNotPresent,
+					PullPolicy:      common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					AllowedServices: []string{common.TestAlpineImage},
 				},
 			},
@@ -821,7 +824,8 @@ func TestDockerImageNameFromVariable(t *testing.T) {
 
 	re := regexp.MustCompile("(?m)^ERROR: The [^ ]+ is not present on list of allowed images")
 
-	output := runTestJobWithOutput(t, build)
+	output, err := buildtest.RunBuildReturningOutput(t, build)
+	require.NoError(t, err)
 	assert.NotRegexp(t, re, output, "Image's name should be expanded from variable")
 }
 
@@ -845,7 +849,7 @@ func TestDockerServiceNameFromVariable(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:           common.TestAlpineImage,
-					PullPolicy:      common.PullPolicyIfNotPresent,
+					PullPolicy:      common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					AllowedServices: []string{common.TestAlpineImage},
 				},
 			},
@@ -854,7 +858,8 @@ func TestDockerServiceNameFromVariable(t *testing.T) {
 
 	re := regexp.MustCompile("(?m)^ERROR: The [^ ]+ is not present on list of allowed services")
 
-	output := runTestJobWithOutput(t, build)
+	output, err := buildtest.RunBuildReturningOutput(t, build)
+	require.NoError(t, err)
 	assert.NotRegexp(t, re, output, "Service's name should be expanded from variable")
 }
 
@@ -1057,7 +1062,7 @@ func testDockerVersion(t *testing.T, version string) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:       common.TestAlpineImage,
-					PullPolicy:  common.PullPolicyIfNotPresent,
+					PullPolicy:  common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					Credentials: credentials,
 					CPUS:        "0.1",
 				},
@@ -1127,7 +1132,7 @@ func TestDockerCommandWithBrokenGitSSLCAInfo(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -1158,7 +1163,7 @@ func TestDockerCommandWithGitSSLCAInfo(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestAlpineImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -1189,7 +1194,7 @@ func TestDockerCommandWithHelperImageConfig(t *testing.T) {
 				Docker: &common.DockerConfig{
 					Image:       common.TestAlpineImage,
 					HelperImage: helperImageConfig,
-					PullPolicy:  common.PullPolicyIfNotPresent,
+					PullPolicy:  common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 				},
 			},
 		},
@@ -1234,7 +1239,7 @@ func TestDockerCommandWithDoingPruneAndAfterScript(t *testing.T) {
 				Executor: "docker",
 				Docker: &common.DockerConfig{
 					Image:      common.TestDockerGitImage,
-					PullPolicy: common.PullPolicyIfNotPresent,
+					PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 					Volumes: []string{
 						"/var/run/docker.sock:/var/run/docker.sock",
 					},
@@ -1516,14 +1521,14 @@ func TestChownAndUmaskUsage(t *testing.T) {
 						Executor: "docker",
 						Docker: &common.DockerConfig{
 							Image:      tt.testImage,
-							PullPolicy: common.PullPolicyIfNotPresent,
+							PullPolicy: common.DockerPullPolicies{common.PullPolicyIfNotPresent},
 						},
 					},
 				},
 			}
 
-			output := runTestJobWithOutput(t, build)
-			fmt.Println(output)
+			output, err := buildtest.RunBuildReturningOutput(t, build)
+			require.NoError(t, err)
 
 			tt.assertOutput(t, output)
 		})
