@@ -53,9 +53,13 @@ func newDefaultManager(config ManagerConfig) *manager {
 	return m
 }
 
-func addParser(manager *manager) *parser.MockParser {
+func addUnixParser(manager *manager) *parser.MockParser {
+	return addParser(manager, path.NewUnixPath())
+}
+
+func addParser(manager *manager, p path.Path) *parser.MockParser {
 	parserMock := new(parser.MockParser)
-	parserMock.On("Path").Return(path.NewUnixPath())
+	parserMock.On("Path").Return(p)
 
 	manager.parser = parserMock
 	return parserMock
@@ -124,7 +128,7 @@ func TestDefaultManager_CreateUserVolumes_HostVolume(t *testing.T) {
 
 			m := newDefaultManager(config)
 
-			volumeParser := addParser(m)
+			volumeParser := addUnixParser(m)
 			defer volumeParser.AssertExpectations(t)
 
 			volumeParser.On("ParseVolume", existingBinding).
@@ -200,7 +204,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_Disabled(t *testing.T) {
 
 			m := newDefaultManager(config)
 
-			volumeParser := addParser(m)
+			volumeParser := addUnixParser(m)
 			defer volumeParser.AssertExpectations(t)
 
 			volumeParser.On("ParseVolume", "/host:/duplicated").
@@ -292,7 +296,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_HostBased(t *testing.T) {
 
 			m := newDefaultManager(config)
 
-			volumeParser := addParser(m)
+			volumeParser := addUnixParser(m)
 			defer volumeParser.AssertExpectations(t)
 
 			volumeParser.On("ParseVolume", existingBinding).
@@ -367,7 +371,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_VolumeBased(t *testing.T)
 			}
 
 			m := newDefaultManager(config)
-			volumeParser := addParser(m)
+			volumeParser := addUnixParser(m)
 			mClient := new(docker.MockClient)
 			m.client = mClient
 
@@ -418,7 +422,7 @@ func TestDefaultManager_CreateUserVolumes_CacheVolume_VolumeBased_WithError(t *t
 	}
 
 	m := newDefaultManager(config)
-	volumeParser := addParser(m)
+	volumeParser := addUnixParser(m)
 	mClient := new(docker.MockClient)
 	m.client = mClient
 
@@ -506,7 +510,7 @@ func TestDefaultManager_CreateTemporary(t *testing.T) {
 			}
 
 			m := newDefaultManager(config)
-			volumeParser := addParser(m)
+			volumeParser := addUnixParser(m)
 			mClient := new(docker.MockClient)
 			m.client = mClient
 
