@@ -2,7 +2,6 @@ package shell_test
 
 import (
 	"bytes"
-	"errors"
 	"fmt"
 	"io/ioutil"
 	"net/http"
@@ -35,7 +34,7 @@ func gitInDir(dir string, args ...string) ([]byte, error) {
 	return cmd.Output()
 }
 
-func skipOnGitWithMessage(t *testing.T, constraints string, message string) {
+func skipOnGit(t *testing.T, constraints string) {
 	out, err := gitInDir("", "version")
 	if err != nil {
 		t.Fatal("Can't detect git version", err)
@@ -66,22 +65,10 @@ func skipOnGitWithMessage(t *testing.T, constraints string, message string) {
 
 	shouldSkip := rules.Check(gitVersion)
 	if shouldSkip {
-		t.Skipf("Git %q found, skipping the test; %s", constraints, message)
+		t.Skipf("Git %q found, skipping the test", constraints)
 	}
 }
 
-func skipIfGitDoesNotSupportLFS(t *testing.T) {
-	skipOnGitWithMessage(t, "< 1.8.2", "available git version doesn't support LFS")
-}
-
-func skipOnGit(t *testing.T, constraints string) {
-	skipOnGitWithMessage(t, constraints, "")
-}
-
-func skipOnGit17x(t *testing.T) {
-	skipOnGit(t, "< 1.8")
-}
-
 func newBuild(t *testing.T, getBuildResponse common.JobResponse, shell string) (*common.Build, func()) {
 	dir, err := ioutil.TempDir("", "gitlab-runner-shell-executor-test")
 	if err != nil {
@@ -192,7 +179,8 @@ func TestMultistepBuild(t *testing.T) {
 
 				if tt.errExpected {
 					var buildErr *common.BuildError
-					assert.True(t, errors.As(err, &buildErr), "expected %T, got %T", buildErr, err)
+					assert.ErrorAs(t, err, &buildErr)
+					assert.Equal(t, 1, buildErr.ExitCode)
 					return
 				}
 				assert.NoError(t, err)
@@ -374,6 +362,25 @@ func TestBuildWithHeadLock(t *testing.T) {
 	})
 }
 
+func TestBuildWithLeftoverConfigLock(t *testing.T) {
+	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
+		successfulBuild, err := common.GetSuccessfulBuild()
+		assert.NoError(t, err)
+		build, cleanup := newBuild(t, successfulBuild, shell)
+		defer cleanup()
+
+		err = buildtest.RunBuild(t, build)
+		assert.NoError(t, err)
+
+		build.JobResponse.AllowGitFetch = true
+		err = ioutil.WriteFile(build.BuildDir+"/.git/config.lock", []byte{}, os.ModeSticky)
+		require.NoError(t, err)
+
+		err = buildtest.RunBuild(t, build)
+		assert.NoError(t, err)
+	})
+}
+
 func TestBuildWithGitLFSHook(t *testing.T) {
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
@@ -443,8 +450,6 @@ func TestBuildWithGitStrategyNoneWithoutLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyNoneWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -486,8 +491,6 @@ func TestBuildWithGitStrategyFetchWithoutLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyFetchWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -513,8 +516,6 @@ func TestBuildWithGitStrategyFetchWithLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyFetchWithUserDisabledLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -575,8 +576,6 @@ func TestBuildWithGitStrategyFetchNoCheckoutWithoutLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyFetchNoCheckoutWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -628,8 +627,6 @@ func TestBuildWithGitStrategyCloneWithoutLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyCloneWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -646,8 +643,6 @@ func TestBuildWithGitStrategyCloneWithLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyCloneWithUserDisabledLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -694,8 +689,6 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithoutLFS(t *testing.T) {
 }
 
 func TestBuildWithGitStrategyCloneNoCheckoutWithLFS(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulLFSBuild()
 		assert.NoError(t, err)
@@ -717,8 +710,6 @@ func TestBuildWithGitStrategyCloneNoCheckoutWithLFS(t *testing.T) {
 }
 
 func TestBuildWithSubmoduleLFSPullsLFSObject(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -742,8 +733,6 @@ func TestBuildWithSubmoduleLFSPullsLFSObject(t *testing.T) {
 }
 
 func TestBuildWithSubmoduleLFSDisabledSmudging(t *testing.T) {
-	skipIfGitDoesNotSupportLFS(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetRemoteSuccessfulBuild()
 		assert.NoError(t, err)
@@ -821,8 +810,6 @@ func TestBuildWithGitSubmoduleStrategyNormal(t *testing.T) {
 }
 
 func TestBuildWithGitSubmoduleStrategyRecursive(t *testing.T) {
-	skipOnGit17x(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
 		assert.NoError(t, err)
@@ -846,8 +833,6 @@ func TestBuildWithGitSubmoduleStrategyRecursive(t *testing.T) {
 }
 
 func TestBuildWithGitFetchSubmoduleStrategyRecursive(t *testing.T) {
-	skipOnGit17x(t)
-
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
 		successfulBuild, err := common.GetSuccessfulBuild()
 		assert.NoError(t, err)
@@ -1024,7 +1009,7 @@ func TestBuildMultilineCommand(t *testing.T) {
 }
 
 func TestBuildWithBrokenGitSSLCAInfo(t *testing.T) {
-	skipOnGit17x(t)
+	skipOnGit(t, "< 1.9")
 	skipOnGit(t, ">= 2.10.2")
 
 	shellstest.OnEachShell(t, func(t *testing.T, shell string) {
@@ -1346,11 +1331,7 @@ func TestBuildFileVariablesRemoval(t *testing.T) {
 
 				_, err := os.Stat(variableFile)
 				assert.Error(t, err)
-				assert.True(
-					t,
-					errors.Is(err, os.ErrNotExist),
-					`Expected that os.Stat on the variable file will return the "doesn't exist" error`,
-				)
+				assert.ErrorIs(t, err, os.ErrNotExist)
 			})
 		})
 	}
