@@ -10,7 +10,19 @@ const (
 	OSTypeLinux   = "linux"
 	OSTypeWindows = "windows"
 
-	name = "gitlab/gitlab-runner-helper"
+	//nolint:lll
+	// DockerHubWarningMessage is the message that is printed to the user when
+	// it's using the helper image hosted in Docker Hub. It is up to the caller
+	// to print this message.
+	DockerHubWarningMessage = "Pulling GitLab Runner helper image from Docker Hub. " +
+		"Helper image is migrating to registry.gitlab.com, " +
+		"for more information see " +
+		"https://docs.gitlab.com/runner/configuration/advanced-configuration.html#migrating-helper-image-to-registrygitlabcom"
+
+	// DockerHubName is the name of the helper image hosted in Docker Hub.
+	DockerHubName = "gitlab/gitlab-runner-helper"
+	// GitLabRegistryName is the name of the helper image hosted in registry.gitlab.com.
+	GitLabRegistryName = "registry.gitlab.com/gitlab-org/gitlab-runner/gitlab-runner-helper"
 
 	headRevision        = "HEAD"
 	latestImageRevision = "latest"
@@ -35,6 +47,7 @@ type Config struct {
 	Architecture    string
 	OperatingSystem string
 	Shell           string // Currently only used by the Docker executor on the Windows platform
+	GitLabRegistry  bool
 }
 
 type creator interface {
@@ -62,3 +75,11 @@ func imageRevision(revision string) string {
 
 	return latestImageRevision
 }
+
+func imageName(gitlabRegistry bool) string {
+	if gitlabRegistry {
+		return GitLabRegistryName
+	}
+
+	return DockerHubName
+}
